lin.models <-
function(depVar, outData, eigSet, covars, pigengeneFile=NULL, verbose=1,
                       outPath=experimentPath, weed=FALSE, saveAllCoefs=TRUE, label=NULL,
                       plots=TRUE, idVal1=idVal){
    ## Fits a linear model per module
    ## outData:A data frame with participants on rows. Covariates and  outcomes on columns.
    ## Row names must be unique IDs for participants.
    ## depVar: A string showing the column in outData that is the dependent variable
    ## (e.g., "total_brain_pct").
    ## eigSet: A data frame in which rows are participants and columns are the eigen proteins.
    ## Output: For each eigenprotein, the summary of the model will be saved in a csv file
    ## in a folder named as the outcome.
    result <- list()
    ## make directory to store results
    dir.create(file.path(outPath, depVar), recursive=TRUE, showWarnings=FALSE)

    ## merge mri and protein data
    tot <- merge(eigSet,outData,by=0)
    result[["Before_excl"]] <- nrow(tot)
    tot2 <- tot[which(!is.na(tot$total_brain_pct)),]
    result[["MR_avail"]] <- nrow(tot2)
    
    ## remove people with Other Neuro Disorders
    tot3<-tot2[which(tot2$mr_excludeon==0),]
    result[["Excl_ON"]] <- nrow(tot3)    
    totnarrow <- tot3[, c(idVal1, depVar, colnames(eigSet), covars) ]

    ## remove people with missing depVar
    totsmall <- na.omit(totnarrow)
    if(verbose>0){
        print(paste("N in", depVar, "analysis"))
        print(nrow(totsmall))
    }
    result[["Excl_missing"]] <- nrow(totsmall)

    summ <- matrix(, nrow=1, ncol=2)
    module2coefs <- list()
    weeded <- list()
    depVarPath <- file.path(outPath, depVar)
    weedingPath <- file.path(depVarPath, "weeding")
    dir.create(weedingPath, recursive=TRUE, showWarnings=FALSE)
    ## For every module:
    for(col1 in colnames(eigSet)){
        if(verbose>1)
            print(col1)
        col <- totsmall[, col1]
        totsmall$zcol1 <- (col - mean(col))/sd(col)
        model1 <- lin.model(depVar=depVar, indepVar="zcol1", covars=covars, Data=totsmall)
        coefs1 <- model1$coefs
        rownames(coefs1)[which(rownames(coefs1)=="zcol1")] <- col1
        module2coefs[[col1]] <- coefs1
        if(saveAllCoefs==TRUE){
            write.csv(coefs1, file = file.path(outPath, depVar,
                                               paste("coefs_", depVar, col1, ".csv", sep = "")))
        }
        summr <- c(model1$adj.rsq, model1$rsq)
        summ <- rbind(summ, summr)
        rownames(summ)[nrow(summ)] <- col1
        if(depVar=="total_brain_pct" & plots==TRUE){
        plot1 <- ggplot(totsmall, aes(x=zcol1, y=total_brain_pct)) + geom_point() +
            geom_smooth(method=lm)
        ggsave(file.path(depVarPath, paste0(col1, ".png")), plot1)
        residsMod <- lin.model(depVar="zcol1", indepVar=NULL, covars=covars, Data=totsmall)
        totsmall$resids <- residsMod$resids
        plot2 <- ggplot(totsmall, aes(x=resids, y=total_brain_pct)) + geom_point() +
            geom_smooth(method=lm)
        ggsave(file.path(depVarPath, paste0(col1, "_resids.png")), plot2)
        }
        
        if(weed==TRUE){
            if(!is.null(pigengeneFile)){
                weeded[[col1]] <- weed.proteins(pigengeneFile=pigengeneFile,
                                                depVar=depVar, indepVar=col1, outData=outData,
                                                covars=covars,
                                                verbose=verbose-1,
                                                outPath=file.path(weedingPath, col1))
            }
        }
    }

if(!is.null(covars)){
    lmtemp <- lin.model(depVar=depVar, indepVar=NULL, covars=covars, Data=totsmall)
    adj.rsqs <- lmtemp$adj.rsq
    rsqs <- lmtemp$rsq
    summr <- c(adj.rsqs, rsqs)
    summ <- rbind(summ, summr)
    rownames(summ)[nrow(summ)] <- "covariates only"
    }
    summ <- as.matrix(round(summ[-1,1:2],3))
    colnames(summ) <- c("adj.r.sq", "r.sq")
    write.csv(summ, file = file.path(outPath, depVar,
                                     paste("rsqs_", depVar, label, ".csv", sep = "") ))
    result[["all.rsqs"]] <- summ
    result[["coefs"]] <- module2coefs
    result[["weeded"]] <- weeded
    result[["samplemod"]] <- model1$samplemod
    result[["Data"]] <- totnarrow
    models <- result
    save(models, file=file.path(depVarPath, paste(depVar, label, "models.RData", sep="_")))
    return(result)
}
