weed.proteins.surv <-
function(depVar, timeVar, indepVar, pigengeneFile, outData, verbose=1, covars,
                          outPath=file.path(dirname(pigengeneFile), "weeding")){
    ## Using a greedy algorithm, we remove proteins one by one, recompute the eigenprotein,
    ##^ fit a linear model to this new eigenprotein, and measure the performance
    ## (based on p-value, R-square, etc.)
    ## This is useful to identify the "core" of a module, i.e., the proteins with the
    ## highest impact.
    ## Data:A data frame with participants on rows. Covariates and  outcomes on columns.
    ## Row names must be unique IDs for participants.
    ## depVar: A string showing the column in Data that is the dependent variable
    ## (e.g., "total_brain_pct").
    ## indevpVar: A string showing the column in Data that is the independent variable
    ## (e.g., "ME1").

    ## Output: The summary of the models,  R-square, etc.
    result <- list()
    pvals <- c()
    models <- list()
    coefs <- list()
    phPvals <- c()
    load(pigengeneFile, v=verbose>0) ## pigengene
    dir.create(outPath, showWarnings=FALSE, recursive=FALSE)
    absWeights <- sort(abs(pigengene$membership[,indepVar]))
    ## proteins in the module
    proteins <- names(which(pigengene$orderedModules==gsub(indepVar, pattern="ME", replacement="")))
    proteins <- names(sort(absWeights[proteins]))
    included <- proteins

    if(verbose>0){
        print(paste("Number of proteins:", length(absWeights)))
    }
    for(p1 in proteins){
        if(verbose > 2)
            print(p1)
        projected1 <- project.eigen(Data=pigengene$Data[ , included], pigengene=pigengene,
                                    saveFile=NULL, verbose=verbose-1)
        projected1 <- projected1$projected

        ## merge mri and protein data
        tot <- merge(projected1, outData, by=0)
        ## Remove participants with missing outcome variable.
        totsmall <- tot[!is.na(tot[[depVar]]), ]
        if(length(included) == length(proteins)) { ## Only in the first iteration,
            if(verbose>0){
                result[["sampleNum"]] <- nrow(tot)  ## Total number of participants
                ## remove people with missing depVar
                print(paste("Number of participants in", depVar, "analysis"))
                print(nrow(totsmall))
                result[["nonNaNum"]] <- nrow(totsmall)
            }
        }

        totsmall[indepVar] <- scale(totsmall[indepVar])
        
        ph.dummy <- function(f1){
            if(!inherits(f1,"formula"))
                stop("f1 must inherit from call formula!"); 
            model <- coxph(f1, data = totsmall);  
            return(model)
        }  

        ## Fit a Cox PH model:       
        model1 <-ph.dummy(
            f1=as.formula(
                if(is.null(covars))
                {paste0("Surv(", timeVar, ",", depVar, ") ~ ",
                        indepVar)}
                else {
                    paste("Surv(", timeVar, ",", depVar, ") ~ ",
                          indepVar, " + ", paste(
                                              covars, collapse = " + "
                                          )
                          )}
            )
        )

        models[[p1]] <- model1 ## The model with p1 and all proteins with a better weight than p1
        coefs[[p1]] <- summary(model1)$coefficients
        phPvals[p1] <- cox.zph(model1)$table["GLOBAL","p"]
        pvals[p1] <- summary(model1)$coefficients[indepVar,"Pr(>|z|)"]
        if(verbose>5)
            print(pvals[p1])
        if(pvals[p1]==min(pvals))
            minEigen <- projected1[,indepVar]
        ## Remove 1 protein:
        included <- names(sort(absWeights[included])[2:length(included)])
        if(length(included) <2)
            break
    }
    ## Plots:
   
    png(file.path(outPath, "pvals.png"))
    plot(y=pvals*4, x=rev(2:(length(pvals)+1)),
         ylab="Adjusted p-value", xlab="Number of proteins",
         xlim=rev(range(2:(length(pvals)+1))), ylim=c(0,0.3))
    abline(h=0.05)
    dev.off()

    result[["minEigen"]] <- minEigen
    result[["proteins"]] <- proteins
    result[["models"]] <- models
    result[["coefs"]] <- coefs
    result[["pvals"]] <- pvals
    result[["phPvals"]] <- phPvals
    weeded <- result
    save(weeded, file=file.path(outPath, "weeded.Rdata"))
    return(result)
}
